"""Methods for obtaining information about the status of the XRP Ledger."""
from xrpl.ledger.main import (
    get_fee,
    get_latest_open_ledger_sequence,
    get_latest_validated_ledger_sequence,
)

__all__ = [
    "get_latest_validated_ledger_sequence",
    "get_fee",
    "get_latest_open_ledger_sequence",
]
